package com.miaozhen.mzanalytics_mybmw;

import android.app.Activity;
import android.content.Context;

import androidx.annotation.NonNull;

import org.json.JSONObject;

import java.util.List;
import java.util.Map;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * MzanalyticsMybmwPlugin
 */
public class MzanalyticsMybmwPlugin implements FlutterPlugin, MethodCallHandler {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private MethodChannel channel;
    private static Context applicationContext;

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "mzanalytics_mybmw");
        channel.setMethodCallHandler(this);
        applicationContext = flutterPluginBinding.getApplicationContext();
    }

    // This static function is optional and equivalent to onAttachedToEngine. It supports the old
    // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
    // plugin registration via this function while apps migrate to use the new Android APIs
    // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
    //
    // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
    // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
    // depending on the user's project. onAttachedToEngine or registerWith must both be defined
    // in the same class.
    @SuppressWarnings("deprecation")
    public static void registerWith(Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), "mzanalytics_mybmw");
        channel.setMethodCallHandler(new MzanalyticsMybmwPlugin());
        Activity activity = registrar.activity();
        applicationContext = activity.getApplicationContext();
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        try {
            if (call.method.equals("initEvent")) {
                EventUtils.initEvent(applicationContext);
            } else if (call.method.equals("trackEvent")) {
                Map<String, String> map = (Map<String, String>) call.arguments;
                JSONObject baseObject = new JSONObject(map);
                EventUtils.trackEvent(applicationContext, baseObject);
            } else if (call.method.equals("trackPageView")) {
                Map<String, String> map = (Map<String, String>) call.arguments;
                JSONObject baseObject = new JSONObject(map);
                EventUtils.trackPageView(applicationContext, baseObject);
            } else if (call.method.equals("trackSiteCustomEvent")) {
                Map<String, String> map = (Map<String, String>) call.arguments;
                JSONObject baseObject = new JSONObject(map);
                EventUtils.trackSiteCustomEvent(applicationContext, baseObject);
            } else if (call.method.equals("trackPulse")) {
                EventUtils.trackPulse(applicationContext);
            } else if (call.method.equals("getCid")) {
                String cid = EventUtils.getCid(applicationContext);
                result.success(cid);
            } else if (call.method.equals("getOaid")) {
                String oaid = EventUtils.getOaid(applicationContext);
                result.success(oaid);
            } else if (call.method.equals("setCd")) {
                String key = call.argument("cd_key");
                String value = call.argument("cd_value");
                EventUtils.setCd(applicationContext, key, value);
            } else if (call.method.equals("setCm")) {
                String key = call.argument("cm_key");
                String value = call.argument("cm_value");
                EventUtils.setCm(applicationContext, key, value);
            } else if (call.method.equals("deleteCd")) {
                String key = call.argument("cd_key");
                EventUtils.deleteCd(applicationContext, key);
            } else if (call.method.equals("deleteCm")) {
                String key = call.argument("cm_key");
                EventUtils.deleteCm(applicationContext, key);
            } else if (call.method.equals("getCdAndCm")) {
                Map getCdAndCm = EventUtils.getCdAndCm(applicationContext);
                result.success(getCdAndCm);
            } else if (call.method.equals("isAutoTrackpage")) {
                boolean isAutoTrackpage = EventUtils.isAutoTrackpage(applicationContext);
                result.success(isAutoTrackpage);
            } else if (call.method.equals("getManualtrackpage")) {
                List blackList = EventUtils.getManualtrackpage(applicationContext);
                result.success(blackList);
            }else if (call.method.equals("handleOpenURL")){
                String linkUrl = (String) call.arguments;
                JSONObject baseObject = new JSONObject();
                baseObject.put("linUrl",linkUrl);
                EventUtils.deepLinkEvent(applicationContext, baseObject);
            }else if (call.method.equals("setEnableLog")){
                boolean isDebug = call.argument("Enable");
                EventUtils.setLogEnabled(applicationContext, isDebug);
            }

            else {
                result.notImplemented();
            }
        } catch (Throwable e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }
}
