import 'dart:async';

import 'package:mzanalytics_mybmw/mzanalytics_log.dart';

class MzanalyticsCheckOut {
  ///正则判断表
  static RegExp number = RegExp(r"\d$");

  ///方法列表
  static Future<bool> appidCheck(var appid) async {
    try {
      if ((appid is String) == false) {
        MzanalyticsLog.p('appid 必须为String类型');
        return false;
      }

      if (number.hasMatch(appid) == false) {
        MzanalyticsLog.p('appid  必须为数字');
        return false;
      }

      return true;
    } catch (e) {
      return false;
    }
  }

  static Future<bool> trackPageViewCheck(var p) async {
    try {
      if ((p is Map) == false) {
        MzanalyticsLog.p('传入参数必须为Map类型');
        return false;
      }

      if ((p['dt'] is String) == false) {
        MzanalyticsLog.p('dt 必须为String类型');
        return false;
      }

      if (p['CustomDimMetrics'] is Map) {
        bool ret = await cdmCheck(p['CustomDimMetrics']);

        return ret;
      }

      return true;
    } catch (e) {
      return false;
    }
  }

  static Future<bool> trackEventCheck(var p) async {
    try {
      if ((p is Map) == false) {
        MzanalyticsLog.p('传入参数必须为Map类型');
        return false;
      }

      if (((p['ec'] is String) == false) &&
          ((p['ea'] is String) == false) &&
          ((p['el'] is String) == false)) {
        MzanalyticsLog.p('请输入事件分类、事件动作、事件标签中的任意一个参数');
        return false;
      }
      if ((p['ev'] != null) && (p['ev'] is String) == true) {
        if (!number.hasMatch(p['ev'])) {
          MzanalyticsLog.p('ev  必须为数字');
          return false;
        }
      }

      if (p['CustomDimMetrics'] is Map) {
        bool ret = await cdmCheck(p['CustomDimMetrics']);
        return ret;
      }

      return true;
    } catch (e) {
      return false;
    }
  }

  static Future<bool> trackSiteCustomEventCheck(var p) async {
    try {
      if ((p is Map) == false) {
        MzanalyticsLog.p('传入参数必须为Map类型');
        return false;
      }

      if ((p['CustomEvent'] is Map) == false) {
        MzanalyticsLog.p('CustomEvent 必须为Map类型');
        return false;
      }

      bool caRet = await caCheck(p['CustomEvent']);

      if (caRet == false) {
        return false;
      }

      if (p['CustomDimMetrics'] is Map) {
        bool ret = await cdmCheck(p['CustomDimMetrics']);
        return ret;
      }

      return true;
    } catch (e) {
      return false;
    }
  }

  //自定义指标判断
  static Future<bool> caCheck(Map p) async {
    try {
      if ((p['caid'] is String) == false) {
        MzanalyticsLog.p('caid 必须为String类型');
        return false;
      }
      if (!number.hasMatch(p['caid'])) {
        MzanalyticsLog.p('caid  必须为数字');
        return false;
      }
      for (var i = 1; i < 21; i++) {
        String calK = 'cal' + i.toString();
        String cavK = 'cav' + i.toString();
        if (p.containsKey(calK)) {
          if ((p[calK] is String) == false) {
            MzanalyticsLog.p(calK + ' 必须为String类型');
            return false;
          }
        }
        if (p.containsKey(cavK)) {
          if ((p[cavK] is String) == false) {
            MzanalyticsLog.p(cavK + ' 必须为String类型');
            return false;
          }
          if (!number.hasMatch(p[cavK])) {
            MzanalyticsLog.p(cavK + '  必须为数字');
            return false;
          }
        }
      }

      return true;
    } catch (e) {
      return false;
    }
  }

//通用的高级维度判断
  static Future<bool> cdmCheck(Map p) async {
    try {
      for (var i = 1; i < 26; i++) {
        String cdK = 'cd' + i.toString();
        String cmK = 'cm' + i.toString();
        if (p.containsKey(cdK)) {
          if ((p[cdK] is String) == false) {
            MzanalyticsLog.p(cdK + ' 必须为String类型');
            return false;
          }
        }
        if (p.containsKey(cmK)) {
          if ((p[cmK] is String) == false) {
            MzanalyticsLog.p(cmK + ' 必须为String类型');
            return false;
          }
          if (!number.hasMatch(p[cmK])) {
            MzanalyticsLog.p(cmK + '  必须为数字');
            return false;
          }
        }
      }

      return true;
    } catch (e) {
      return false;
    }
  }

  static Future<bool> setCdCheck(var cd) async {
    try {
      if ((cd is String) == false) {
        MzanalyticsLog.p('cd 的 Key 与 Value 必须为String类型');
        return false;
      }

      return true;
    } catch (e) {
      return false;
    }
  }

  static Future<bool> setCmCheck(var cm) async {
    try {
      if ((cm is String) == false) {
        MzanalyticsLog.p('cm 必须为String类型');
        return false;
      }
      if (!number.hasMatch(cm)) {
        MzanalyticsLog.p('cm  必须为数字');
        return false;
      }

      return true;
    } catch (e) {
      return false;
    }
  }
}
